﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CSVDataEngine;
using dox42.CustomDataSourceKit;
using dox42.CustomDataSourceKit.RemoteController;

namespace CSVConfigDialog
{
    public partial class ConfigCSVDataSource : Form, IDataSourceConfigDialog
    {
        public ConfigCSVDataSource()
        {
            InitializeComponent();
            txtDSName.Text = dox42RemoteController.Instance.getAddInRC().getProposedNameForDataSource("CSV");
            this.AcceptButton = btnOK;
        }

        /// <summary>
        /// Create the Entry in the Datamap
        /// TODO 7: Write GUI values to command string
        /// </summary>
        /// <returns></returns>
        public string getCommandString()
        {
            return CSVDataSourceParser.buildCommandString(txtDSName.Text, txtFile.Text, rbFixedFile.Checked, lblFilePathDataField.Text, txtUserName.Text, txtPassword.Text);
        }

        
        /// <summary>
        /// setCommandString will be callend from the Datamap Designer, when opening the 
        /// Dialog for an already configured Datasource
        /// </summary>
        /// <param name="commandString"></param>
        public void setCommandString(string commandString )
        {
            IParserRC parser = dox42RemoteController.Instance.getParserRC (commandString);
            txtDSName.Text = parser.getDataSourceName();
            txtFile.Text = parser.getElementValue(CSVDataSourceParser.FileName, true);
            lblFilePathDataField.Text = parser.getElementValue(CSVDataSourceParser.DataFieldFilePath, true);
            
            rbFixedFile.Checked= true;
            string sUseFixeFilePath = parser.getElementValue(CSVDataSourceParser.UseFixedFilePath, true).ToLower().Trim();
            if (!string.IsNullOrEmpty(sUseFixeFilePath))
                rbFixedFile.Checked = Boolean.Parse(sUseFixeFilePath);
            
            rbDataFieldFilePath.Checked = !rbFixedFile.Checked;

            txtUserName.Text = parser.getElementValue(CSVDataSourceParser.UserName, true);
            txtPassword.Text = parser.getElementValue(CSVDataSourceParser.Password, true);
        }

        private void btnPickCSVFile_Click(object sender, EventArgs e)
        {
            try
            {
                OpenFileDialog o = new OpenFileDialog();
                o.Title = "Pick CSV File";
                if (o.ShowDialog() == DialogResult.OK)
                {
                    txtFile.Text = o.FileName;
                }
                greystate();
            }
            catch (Exception exc)
            {
                dox42RemoteController.Instance.handleError(exc);
            }
        }

        private void txtDSName_TextChanged(object sender, EventArgs e)
        {
            greystate();
        }

        /// <summary>
        /// Check the Characters allowed for a DataSource Name
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtDSName_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = !Char.IsLetterOrDigit(e.KeyChar) && e.KeyChar != '_' && e.KeyChar != '-' && !Char.IsControl(e.KeyChar);
            greystate();
        }

        private void txtFile_TextChanged(object sender, EventArgs e)
        {
            greystate();
        }

        private void ConfigCSVDataSource_Load(object sender, EventArgs e)
        {
            greystate();
        }

        private void greystate()
        {
            btnOK.Enabled = txtDSName.Text != "" && 
                            ((rbFixedFile.Checked && txtFile.Text != "") ||
                            (rbDataFieldFilePath.Checked && lblFilePathDataField.Text != ""));
            
            btnTest.Enabled = btnOK.Enabled;
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            Close();
        }

        /// <summary>
        /// Test your Datasource, open a Datapreview Dialog
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnTest_Click(object sender, EventArgs e)
        {
            IAddInRC addInRC = dox42RemoteController.Instance.getAddInRC();
            try
            {
                CSVDataSource ds = new CSVDataSource();

                ds.Name = txtDSName.Text;
                ds.FixedFilePath = txtFile.Text;
                ds.DataFieldFilePath = lblFilePathDataField.Text;
                ds.UseFixedFilePath = rbFixedFile.Checked;
                
                addInRC.showDataPreviewDialog(ds.GetDataTable());
            }
            catch (Exception exc)
            {
                dox42RemoteController.Instance.handleError(exc);
            }
        }

        
        /// <summary>
        /// Choose a Datafield, open the Datafield Explorer of dox42
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPickDF_Click(object sender, EventArgs e)
        {
            try
            {
                lblFilePathDataField.Text = dox42RemoteController.Instance.getAddInRC().showDataFieldPicker();
            }
            catch (Exception exc)
            {
                dox42RemoteController.Instance.handleError(exc);
            }
        }

        

        private void rbFixedFile_CheckedChanged(object sender, EventArgs e)
        {
            greystate();
        }

        private void lblFilePathDataField_TextChanged(object sender, EventArgs e)
        {
            greystate();
        }

        private void btnPickDFUserName_Click(object sender, EventArgs e)
        {
            InsertDF(txtUserName, true);
        }

        private void btnPickDFPassWord_Click(object sender, EventArgs e)
        {
            InsertDF(txtPassword, true);
        }

        private static void insertStringInTextBox(TextBox txtBox, string toInsert)
        {
            if (toInsert == "") return;

            if (txtBox.SelectedText != null)
            {
                int oldStartSelection = txtBox.SelectionStart;
                txtBox.SelectedText = toInsert;

                txtBox.Focus();
                txtBox.SelectionStart = oldStartSelection + toInsert.Length;
            }
            else
                txtBox.Text = toInsert;
        }

        private void InsertDF(TextBox txtBox, bool showCryptoFields = false)
        {
            try
            {
                string df = "";

                if (showCryptoFields)
                {
                    try
                    {
                        df = ShowDFPickerCrypto();
                    }
                    catch (MissingMethodException) // Ensure compatibility to old dox42 Versions
                    {
                        df = ShowDFPickerNoCrypto();
                    }
                }
                else
                {
                    df = ShowDFPickerNoCrypto();
                }

                if (!string.IsNullOrEmpty(df))
                    insertStringInTextBox(txtBox, "<%" + df + "%>");
            }
            catch (Exception) { }
            finally
            {
                greystate();
            }// Gulp
        }

        private static string ShowDFPickerNoCrypto()
        {
            return dox42RemoteController.Instance.getAddInRC().showDataFieldPicker();
        }

        private static string ShowDFPickerCrypto()
        {
            return dox42RemoteController.Instance.getAddInRC().showDataFieldPickerWithCryptoDataSources();
        }

    }
}
