﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.IO;
using System.Net;
using dox42.CustomDataSourceKit;
using dox42.CustomDataSourceKit.RemoteController;

namespace CSVDataEngine
{
    public class CSVDataSource : IDataSource
    {
        public string Name { get; set; }
        
        // TODO 1: Define your configuration parameters as public properties
        public string FixedFilePath { get; set; }
        public string DataFieldFilePath { get; set; }
        public bool UseFixedFilePath { get; set; } // false -> DataFieldFilePath

        public string Username { get; set; }
        public string Password { get; set; }

        /// <summary>
        /// Inits the Columns of the Datasource
        /// </summary>
        public void initColumns()
        {
            DataTable dataTable = GetDataTable();
            IDataSourceRC dsRC = dox42RemoteController.Instance.getDataSourceRC(Name);
            foreach (DataColumn col in dataTable.Columns)
            {
                dsRC.addColumn(col.ColumnName.ToLower());
            }
        }

        /// <summary>
        /// Loads the Datasource
        /// </summary>
        public void loadData()
        {
            IDataSourceRC dsRC = dox42RemoteController.Instance.getDataSourceRC(Name);
            dsRC.loadData(GetDataTable());
        }

        
        protected string calcFilePath()
        {
            string path = "";

            if (UseFixedFilePath)
                path = FixedFilePath;
            else
            {
                if (DataFieldFilePath.Contains("."))    // not an Inputparameter, but a Datafield of a Datasource
                {
                    string dataSourceName = DataFieldFilePath.Substring(0, DataFieldFilePath.IndexOf('.'));
                    dox42RemoteController.Instance.getDataSourceRC(this.Name).registerAsDependendFrom(dataSourceName);
                }

                path = dox42RemoteController.Instance.getDataSourceRC().getDataFieldValue(DataFieldFilePath);
            }

            return dox42RemoteController.Instance.getCoreRC().replaceDataFieldsInString(path);
        }

        /// <summary>
        /// Returns the content of a CSV File as a Datatable
        /// TODO 2: Implement your logic for data read
        /// </summary>
        /// <returns></returns>
        public DataTable GetDataTable()
        {
            string path = calcFilePath();
            char separator = ';';

            DataTable dt = new DataTable();

            StreamReader sr = null;

            try
            {
                if (path.ToLower().StartsWith("http"))  //URL
                {
                    WebClient wc = new WebClient();

                    if (!string.IsNullOrEmpty(Username) && !string.IsNullOrEmpty(Password))
                    {
                        wc.Credentials = CreateCredentials();
                    }

                    string csvfile = wc.DownloadString(path);
                    sr = new StreamReader(new MemoryStream(System.Text.Encoding.UTF8.GetBytes(csvfile)));
                }
                else
                {
                    FileStream aFile = new FileStream(path, FileMode.Open);
                    sr = new StreamReader(aFile, System.Text.Encoding.Default);
                }

                string strLine = sr.ReadLine();
                string[] strArray = strLine.Split(separator);

                foreach (string value in strArray)
                {
                   dt.Columns.Add(value.Trim());
                }

                DataRow dr = dt.NewRow();

                while (sr.Peek() > -1)
                {
                    strLine = sr.ReadLine();
                    strArray = strLine.Split(separator);
                    dt.Rows.Add(strArray);
                }
                
                return dt;
            }
            finally // No Catch, Exception is handled by dox42
            {
                if (sr != null)
                    sr.Close();
            }
        }

        private NetworkCredential CreateCredentials()
        {
            NetworkCredential creds = new NetworkCredential();
            try
            {
                fillCredsCrypto(Username, Password, creds);
            }
            catch (MissingMethodException) // Ensure compatibility to old dox42 Versions
            {
                fillCredsNoCrypto(Username, Password, creds);
            }
            return creds;
        }

        private static void fillCredsNoCrypto(string UserName, string PassWord, NetworkCredential creds)
        {
            creds.UserName = dox42RemoteController.Instance.getCoreRC().replaceDataFieldsInString(UserName).Trim();
            creds.Password = dox42RemoteController.Instance.getCoreRC().replaceDataFieldsInString(PassWord).Trim();
        }

        private static void fillCredsCrypto(string UserName, string PassWord, NetworkCredential creds)
        {
            creds.UserName = dox42RemoteController.Instance.getCoreRC().replaceCryptoDataFieldsInString(UserName).Trim();
            creds.Password = dox42RemoteController.Instance.getCoreRC().replaceCryptoDataFieldsInString(PassWord).Trim();
        }
    }
}
