﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using dox42.CustomDataSourceKit;
using dox42.CustomDataSourceKit.RemoteController;

namespace CSVDataEngine
{
    public class CSVDataSourceParser :IParser
    {
        // TODO 3: Define XML Names for your configuration parameters
        public const string FileName = "filename";
        public const string DataFieldFilePath = "datafieldfilepath";
        public const string UseFixedFilePath = "usefixedfilepath";
        public const string UserName = "csv_username";
        public const string Password = "csv_password";
        
        private string commandString = "";
        
        public CSVDataSourceParser() { }

        public void setCommandString(string commandString)
        {
            this.commandString = commandString;
        }

        /// <summary>
        /// Create an Entry in the Datamap to save the configuration of the Datasource
        /// TODO 4: alter to write your configuration parameters to command string
        /// </summary>
        /// <param name="name"></param>
        /// <param name="filename"></param>
        /// <param name="useFixedFilePath"></param>
        /// <param name="dataFieldFilePath"></param>
        /// <returns></returns>
        public static string buildCommandString(string name, string filename, bool useFixedFilePath, string dataFieldFilePath, string username, string pw)
        {
            IParserRC parserRC = dox42RemoteController.Instance.getParserRC();
            parserRC.createCommandStringStart(name, typeof(CSVDataSourceParser));
            parserRC.insertElementValue(FileName, filename, true);
            parserRC.insertElementValue(UseFixedFilePath, useFixedFilePath.ToString().ToLower(), false);
            parserRC.insertElementValue(DataFieldFilePath, dataFieldFilePath, false);
            parserRC.insertElementValue(UserName, username, true);
            parserRC.insertElementValue(Password, pw, true);
            parserRC.createCommandStringEnd();

            return parserRC.CommandString;
        }

        /// <summary>
        /// creates a Datasource an initializes the properties of the Datasource from the configuration saved in the Datamap
        /// TODO 5: extract configuration parameters from command string and initialize data source
        /// </summary>
        /// <returns></returns>
        public IDataSource createAndInitDataSource()
        {
            CSVDataSource ds = new CSVDataSource();
            IParserRC parser = dox42RemoteController.Instance.getParserRC(commandString);
            
            ds.UseFixedFilePath = true;
            string sUseFixeFilePath = parser.getElementValue(UseFixedFilePath, true).ToLower().Trim();
            if (!string.IsNullOrEmpty(sUseFixeFilePath))
                ds.UseFixedFilePath = Boolean.Parse(sUseFixeFilePath);
            
            ds.FixedFilePath = parser.getElementValue(FileName, !ds.UseFixedFilePath);
            ds.DataFieldFilePath = parser.getElementValue(DataFieldFilePath, ds.UseFixedFilePath);
            ds.Username = parser.getElementValue(UserName, true);
            ds.Password = parser.getElementValue(Password, true);
            
            return ds;    
        }

        /// <summary>
        /// Which Class is the configuration dialog to be opend in the dox42 Datamap Editor
        /// TODO 9: Insert the class name of your configuraion dialog
        /// </summary>
        /// <returns></returns>
        public string getConfigDialogClassName()
        {
            return "CSVConfigDialog.ConfigCSVDataSource";
        }

        
    }
}
